@@ -84,6 +84,7 @@ module BPACK_DEFS
     integer, parameter :: vec_oversample = 5 !< number of extra vectors adding onto estimated rank in the randomized scheme
     integer, parameter:: msg_chunk = 100000 !< used to determine message tag and hence the massage size
     integer, parameter:: Ndim_max = 100 !< maximum dimension of the inputs
+    integer, parameter:: Nsample_min_MD = 4 !< minimum number of subsamples in MD construction, useful whenrank at previous level or leaf size is too small
 
     !>**** parameters for CEM
     real(kind=8), parameter :: BPACK_cd = 299792458d0 !< free-space speed of light
@@ -162,6 +163,8 @@ module BPACK_DEFS
         integer:: size=0 !< size of the message along first dimension
         integer:: size_i=0 !< size of the message along first dimension
         integer:: active=0 !< whether this communication pair is active
+        integer,allocatable:: offset_md(:) !< offset in my local array (tensor)
+        integer,allocatable:: size_md(:) !< size of the message along first dimensions (tensor)
         DT, allocatable::dat(:, :) !< communication buffer
         integer, allocatable::dat_i(:, :) !< communication buffer
         type(dat_pack), allocatable::dat_pk(:, :) !< communication buffer
@@ -192,14 +195,17 @@ module BPACK_DEFS
     end type nil_onelevel_MD
 
 
-    !>**** input and output vectors for applying a Bplus
+    !>**** input and output vectors for applying a Bplus at one level
     type vectorsblock
-        ! integer style
-        ! integer head
-        ! integer tail
         DT, allocatable :: vector(:, :)
     end type vectorsblock
 
+    !>**** input and output vectors for applying a Bplus at all levels
+    type vectorsblock_oneL
+        type(vectorsblock), allocatable :: vs(:)
+    end type vectorsblock_oneL
+
+
     !>**** a vector used to extract one element of a butterfly
     type vectorset
         DT, allocatable :: vector(:)
@@ -621,6 +627,7 @@ integer, allocatable::index_MD(:, :, :) !< an array of block offsets
         integer:: use_zfp  !< 1: use zfp for the dense blocks (zfp must be used to install ButterflyPACK) 0: do not use zfp
 
         ! options for matrix construction
+        integer Hextralevel !< extra levels for top partitioning of the H matrix based on MPI counts.
         integer forwardN15flag !< 1 use N^1.5 algorithm. 0: use NlogN pseudo skeleton algorithm. 2: use NlogN first, if not accurate enough, switch to N^1.5
         real(kind=8) tol_comp      !< matrix construction tolerance
         integer::Nmin_leaf !< leaf sizes of HODLR tree
@@ -636,12 +643,12 @@ integer, allocatable::index_MD(:, :, :) !< an array of block offsets
         integer:: elem_extract !< 1: use user-defined element extraction 0: use user-defined formula
         integer:: cpp !< 1: use user-defined c/cpp functions 0: use user-defined fortran functions
         integer:: knn !< number of nearest neighbour points for each point
-integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly sample the rows of the unfolded matrices on top of 0. 2: use translation invariance
+        integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly sample the rows of the unfolded matrices on top of 0. 2: use translation invariance
 
         ! options for inversion
         real(kind=8) tol_LS       !< tolerance in pseudo inverse
         real(kind=8) tol_Rdetect  !< tolerance to detect numerical ranks
-        real(kind=8) tol_rand     !< tolerance for randomized contruction
+        real(kind=8) tol_rand     !< tolerance for randomized contruction, mostly used in matrix inversion
         real(kind=8) jitter     !< jittering for dense diagonal blocks
         integer powiter     !< order of power iteration in randomized LR
         integer less_adapt     !< 0 for rank adaptation for all BF levels, 1 for rank adaptation for the outtermost BF levels
@@ -710,13 +717,17 @@ integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly
         procedure(F_Compressibility), nopass, pointer :: FuncNearFar => null() !< Kernels Defined in Fortran: procedure pointer to the user-supplied derived type for determining whether a block in Z (after permutation) is compressible or not
         procedure(F_Zelem_block), nopass, pointer :: FuncZmnBlock => null() !< Kernels Defined in Fortran: procedure pointer to the user-supplied derived type for computing a list of intersection of indices from Z (data layout needs to be provided)
         procedure(F_HMatVec), nopass, pointer :: FuncHMatVec => null() !< Kernels Defined in Fortran: procedure pointer to the user-supplied derived type for computing matvec of Z
+        procedure(F_HMatVec_MD), nopass, pointer :: FuncHMatVec_MD => null() !< Kernels Defined in Fortran: procedure pointer to the user-supplied derived type for computing matvec of Z
 
         type(c_ptr), pointer :: C_QuantApp => null() !< Kernels Defined in C: c_pointer to the user-supplied object for computing one element of Z
         type(c_funptr), pointer :: C_FuncZmn => null() !< Kernels Defined in C: c_function_pointer to the user-supplied function for computing one element of Z
+        type(c_funptr), pointer :: C_FuncZmn_MD => null() !< Kernels Defined in C: c_function_pointer to the user-supplied function for computing one element of Z (as tensors)
         type(c_funptr), pointer :: C_FuncDistmn => null() !< Kernels Defined in C: c_function_pointer to the user-supplied function for computing distance between one row and one column
         type(c_funptr), pointer :: C_FuncNearFar => null() !< Kernels Defined in C: c_function_pointer to the user-supplied function for determine whether a block in Z is compressible or not
+        type(c_funptr), pointer :: C_FuncNearFar_MD => null() !< Kernels Defined in C: c_function_pointer to the user-supplied function for determine whether a block in Z (as tensors) is compressible or not
         type(c_funptr), pointer :: C_FuncZmnBlock => null() !< Kernels Defined in C: c_function_pointer to the user-supplied function for computing a list of intersection of indices from Z (data layout needs to be provided)
         type(c_funptr), pointer :: C_FuncHMatVec => null() !< Kernels Defined in C: procedure pointer to the user-supplied derived type for computing matvec of Z
+        type(c_funptr), pointer :: C_FuncHMatVec_MD => null() !< Kernels Defined in C: procedure pointer to the user-supplied derived type for computing matvec of Z
         type(c_funptr), pointer :: C_FuncBMatVec => null() !< Kernels Defined in C: procedure pointer to the user-supplied derived type for computing matvec of a block
     end type kernelquant
 
@@ -831,6 +842,17 @@ integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly
             CBIND_DT::val
         end subroutine C_Zelem
 
+
+        !> interface of user-defined element evaluation routine in C. m,n represents indices in natural order
+        subroutine C_Zelem_MD(Ndim, m, n, val, quant)
+            USE, INTRINSIC :: ISO_C_BINDING
+            type(c_ptr) :: quant
+            integer(kind=C_INT), INTENT(IN):: Ndim
+            integer(kind=C_INT), INTENT(IN):: m(Ndim), n(Ndim)
+            CBIND_DT::val
+        end subroutine C_Zelem_MD
+
+
         !> interface of user-defined distance computation routine in C. m,n represents indices in natural order
         subroutine C_Dist(m, n, val, quant)
             USE, INTRINSIC :: ISO_C_BINDING
@@ -847,6 +869,16 @@ integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly
             integer(kind=C_INT)::val
         end subroutine C_Compressibility
 
+        !> interface of user-defined distance compressibility routine in C. groupm,groupn represents groups (multi-dimensional) in the permuted order
+        subroutine C_Compressibility_MD(Ndim,groupm, groupn, val, quant)
+            USE, INTRINSIC :: ISO_C_BINDING
+            type(c_ptr) :: quant
+            integer, INTENT(IN):: Ndim
+            integer(kind=C_INT), INTENT(IN):: groupm(Ndim), groupn(Ndim)
+            integer(kind=C_INT)::val
+        end subroutine C_Compressibility_MD
+
+
         !> interface of user-defined element extraction routine in C. allrows,allcols represents indices in natural order
         subroutine C_Zelem_block(Ninter, Nallrows, Nallcols, Nalldat_loc, allrows, allcols, alldat_loc, rowidx, colidx, pgidx, Npmap, pmaps, quant)
             USE, INTRINSIC :: ISO_C_BINDING
@@ -868,6 +900,18 @@ integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly
             character trans
         end subroutine HMatVec
 
+        subroutine HMatVec_MD(Ndim, trans, M, N, num_vect, Vin, Vout, ker)
+            import::mesh, kernelquant, proctree, Hstat
+            implicit none
+            integer Ndim
+            integer, INTENT(IN):: M(Ndim), N(Ndim), num_vect
+            DT::Vin(:, :), Vout(:, :)
+            type(kernelquant)::ker
+            character trans
+        end subroutine HMatVec_MD
+
+
+
         !> interface of user-defined HODLR/H MatVec routine in Fortran.
         subroutine F_HMatVec(trans, M, N, num_vect, Vin, Vout, quant)
             import::mesh, proctree, Hstat
@@ -877,6 +921,18 @@ integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly
             character trans
         end subroutine F_HMatVec
 
+
+        !> interface of user-defined HODLR/H MatVec (tensor) routine in Fortran.
+        subroutine F_HMatVec_MD(Ndim,trans, M, N, num_vect, Vin, Vout, quant)
+            import::mesh, proctree, Hstat
+            integer Ndim
+            class(*), pointer :: quant
+            integer, INTENT(IN):: M(Ndim), N(Ndim), num_vect
+            DT::Vin(:, :), Vout(:, :)
+            character trans
+        end subroutine F_HMatVec_MD
+
+
         !> interface of user-defined HODLR/H MatVec routine in C.
         subroutine C_HMatVec(trans, Nin, Nout, num_vect, Vin, Vout, quant)
             USE, INTRINSIC :: ISO_C_BINDING
@@ -890,6 +946,21 @@ integer:: fastsample_tensor !< 0: uniformly sample each dimension. 1: uniformly
             character(kind=c_char, len=1) :: trans(*)
         end subroutine C_HMatVec
 
+
+        !> interface of user-defined HODLR/H MatVec (tensor) routine in C.
+        subroutine C_HMatVec_MD(Ndim, trans, Nin, Nout, num_vect, Vin, Vout, quant)
+            USE, INTRINSIC :: ISO_C_BINDING
+            import::mesh, proctree, Hstat
+            integer Ndim
+            type(c_ptr) :: quant
+            integer(kind=C_INT), INTENT(IN):: Nin(Ndim), Nout(Ndim), num_vect
+            CBIND_DT::Vin(product(Nin), num_vect), Vout(product(Nout), num_vect)
+            type(mesh)::msh(Ndim)
+            type(proctree)::ptree
+            type(Hstat)::stats
+            character(kind=c_char, len=1) :: trans(*)
+        end subroutine C_HMatVec_MD
+
         !> interface of user-defined Block MatVec routine in C.
         subroutine C_BMatVec(trans, Nin, Nout, num_vect, Vin, Vout, quant, a, b)
             USE, INTRINSIC :: ISO_C_BINDING
