@@ -466,6 +466,7 @@ contains
       implicit none
       type(Hstat)::stats
 
+      if (allocated(stats%leafs_of_level)) deallocate (stats%leafs_of_level)
       if (allocated(stats%rankmax_of_level)) deallocate (stats%rankmax_of_level)
       if (allocated(stats%rankmin_of_level)) deallocate (stats%rankmin_of_level)
       if (allocated(stats%rankmax_of_level_global)) deallocate (stats%rankmax_of_level_global)
@@ -744,6 +745,9 @@ contains
       call MPI_ALLREDUCE(stats%Flop_C_Mult, rtemp, 1, MPI_DOUBLE_PRECISION, MPI_SUM, ptree%Comm, ierr)
       if (ptree%MyID == Main_ID) write (*, '(A21,Es14.2)') 'C_mult flops:', rtemp
 
+      call MPI_ALLREDUCE(stats%Time_RedistV, rtemp, 1, MPI_DOUBLE_PRECISION, MPI_MAX, ptree%Comm, ierr)
+      if(ptree%MyID==Main_ID)write(*,'(A21,Es14.2,A8)') 'RedistV time: ', rtemp, 'Seconds'
+
       call MPI_ALLREDUCE(stats%Time_C_Extract, rtemp, 1, MPI_DOUBLE_PRECISION, MPI_MAX, ptree%Comm, ierr)
       if (ptree%MyID == Main_ID) write (*, '(A21,Es14.2,A8)') 'C_extract time:', rtemp, 'Seconds'
       call MPI_ALLREDUCE(stats%Flop_C_Extract, rtemp, 1, MPI_DOUBLE_PRECISION, MPI_SUM, ptree%Comm, ierr)
@@ -782,6 +786,7 @@ contains
       option%RecLR_leaf = BACA
       option%nogeo = 0
       option%per_geo = 0
+      option%hextralevel = 0
       option%periods = 0
       option%ErrSol = 0
       option%LR_BLK_NUM = 1
@@ -808,7 +813,7 @@ contains
       option%elem_extract = 0
       option%knn = 0
       option%cpp = 0
-      option%bp_cnt_lr = 0
+      option%bp_cnt_lr = 1
       option%less_adapt = 0
 
    end subroutine SetDefaultOptions
@@ -876,6 +881,8 @@ contains
                   read (strings1, *) option%nogeo
                else if (trim(strings) == '--per_geo') then
                   read (strings1, *) option%per_geo
+               else if (trim(strings) == '--hextralevel') then
+                  read (strings1, *) option%hextralevel
                else if (trim(strings) == '--period1') then
                   read (strings1, *) option%periods(1)
                else if (trim(strings) == '--period2') then
@@ -973,6 +980,7 @@ contains
       option1%RecLR_leaf = option%RecLR_leaf
       option1%nogeo = option%nogeo
       option1%per_geo = option%per_geo
+      option1%hextralevel = option%hextralevel
       option1%periods = option%periods
       option1%ErrSol = option%ErrSol
       option1%LR_BLK_NUM = option%LR_BLK_NUM
@@ -1030,6 +1038,7 @@ option1%fastsample_tensor = option%fastsample_tensor
          write (*, '(A18,I8)') 'reclr_leaf', option%RecLR_leaf
          write (*, '(A18,I8)') 'nogeo', option%nogeo
          write (*, '(A18,I8)') 'per_geo', option%per_geo
+         write (*, '(A18,I8)') 'hextralevel', option%hextralevel
          write (*, '(A18,Es14.7)') 'period1', option%periods(1)
          write (*, '(A18,Es14.7)') 'period2', option%periods(2)
          write (*, '(A18,Es14.7)') 'period3', option%periods(3)
